// QStringSyntax.cpp: implementation of the QStringSyntax class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Query32.h"
#include "QAttr.h"
#include "QStringSyntax.h"

#include <string.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// T61 conversion codes
static unsigned char escapes[16][17] = {
        'A','C','E','I','N','O','U','Y', 
        'a','c','e','i','n','o','u','y', ' ',

/* Grave upper case */
        0xc0, '?', 0xc8, 0xcc, '?', 0xd2, 0xd9, '?',  
/* Grave lower case */
        0xe0, '?', 0xe8, 0xec, '?', 0xf2, 0xf9, '?', '`', 

/* Acute upper case */
        0xc1, '?', 0xc9, 0xcd, '?', 0xd3, 0xda, 0xdd,
/* Acute lower case */
        0xe1, '?', 0xe9, 0xed, '?', 0xf3, 0xfa, 0xfd, '\'',

/* Circumflex upper case */
        0xc2, '?', 0xca, 0xce, '?', 0xd4, 0xdb, '?', 
/* Circumflex lower case */
        0xe2, '?', 0xea, 0xee, '?', 0xf4, 0xfb, '?', '?',

/* Tilde upper case */
        0xc3, '?', 0xcb, 0xcf, 0xd1, 0xd5, 0xdc, '?', 
/* Tilde lower case */
        0xe3, '?', 0xeb, 0xef, 0xf1, 0xf5, 0xfc, '?', '?',

/* Macron upper case */
        '?', '?', '?', '?', '?', '?', '?', '?',   
/* Macron lower case */
        '?', '?', '?', '?', '?', '?', '?', '?',  '?', 

/* ? upper case */
        '?', '?', '?', '?', '?', '?', '?', '?', 
/* ? lower case */
        '?', '?', '?', '?', '?', '?', '?', '?',  '?',

/* Dot upper case */
        0xc5, '?', '?', '?', '?', '?', '?', '?',
/* Dot lower case */
        0xe5, '?', '?', '?', '?', '?', '?', '?', '?', 

/* Diaresis upper case */
        0xc4, '?', 0xcb, 0xcf, '?', 0xd6, 0xdc, 0x82,
/* Diaresis lower case */
        0xe4, '?', 0xeb, 0xef, '?', 0xf6, 0xfc, 0xff, '"',

/* Umlaut upper case */
        0xc4, '?', 0xcb, 0xcf, '?', 0xd6, 0xdc, 0x82,
/* Umlaut lower case */
        0xe4, '?', 0xeb, 0xef, '?', 0xf6, 0xfc, 0xff, '"',

/* Ring upper case */
        0xc5, '?', '?', '?', '?', '?', '?', '?', 
/* Ring lower case */
        0xe5, '?', '?', '?', '?', '?', '?', '?', '?',

/* Cedilla upper case */
        '?', 0xc7, '?', '?', '?', '?', '?', '?',
/* Cedilla lower case */
        '?', 0xe7, '?', '?', '?', '?', '?', '?',  '?', 

/* Underline upper case */
        '?', '?', '?', '?', '?', '?', '?', '?',
/* Underline lower case */
        '?', '?', '?', '?', '?', '?', '?', '?',  '_', 

/* Umlaut upper case */
        0xc4, '?', 0xcb, 0xcf, '?', 0xd6, 0xdc, 0x82,

/* Umlaut lower case */
        0xe4, '?', 0xeb, 0xef, '?', 0xf6, 0xfc, 0xff, '"',

/* Cedilla? upper case */
        '?', 0xc7, '?', '?', '?', '?', '?', '?',
/* Cedilla? lower case */
        '?', 0xe7, '?', '?', '?', '?', '?', '?',  '?', 

/* ? upper case */
        '?', '?', '?', '?', '?', '?', '?', '?',
/* ? lower case */
        '?', '?', '?', '?', '?', '?', '?', '?', '?'
};

static unsigned char trans[32] = {
        '?', 0xc6, 0xd0, '?', '?', '?', '?', 'L', 
        'L', 0xd8, '?', '?', 0xde, 'T', 'N', 'n',
        'K', 0xe6, 'd', 0xf0, 'h', 'i', '?', 'l',
        'l', 0xf8, '?', 0xdf, 0xfe, 't', 'N', '?'};


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

QStringSyntax::QStringSyntax( CString &name_arg ) : QSyntax( name_arg )
{
  WIN32_GUFF();
}

QStringSyntax::~QStringSyntax()
{
  WIN32_GUFF();
}

//////////////////////////////////////////////////////////////////////
// Print user friendly value
//////////////////////////////////////////////////////////////////////

void QStringSyntax::Print( CString &str, char *ldap_val_ptr, unsigned long len )
{
	str = "";

	//
	// Perform T.61 conversion
	char *val_ptr = ldap_val_ptr;
  while ( *val_ptr != '\0' )
    {
      if ( ( *val_ptr > 0x1f ) && ( *val_ptr < 0x80 ) )
				{
					str += *val_ptr++;
				}
      else if ((*val_ptr>0xa0) && (*val_ptr < 0xc0))
				{ 
					str += *val_ptr++;
				}
      else if (*val_ptr > 0xdf)
				{
					str += trans[((int)(*val_ptr++)) - 0xe0];
				}
      else if ((*val_ptr > 0xbf) && (*val_ptr < 0xd0))
				{
					int n = (int) (*val_ptr - (unsigned char) 0xc0);
					val_ptr++;

					if (*val_ptr == '\0')
						{
							break;
						}

					switch (*val_ptr)
						{
						case 'A':
							str += escapes[n][0];
							break;

						case 'C':
							str += escapes[n][1];
							break;

						case 'E':
							str += escapes[n][2];
							break;

						case 'I':
							str += escapes[n][3];
							break;

						case 'N':
							str += escapes[n][4];
							break;

						case 'O':
							str += escapes[n][5];
							break;
							
						case 'U':
							str += escapes[n][6];
							break;
							
						case 'Y':
							str += escapes[n][7];
							break;
							
						case 'a':
							str += escapes[n][8];
							break;

						case 'c':
							str += escapes[n][9];
							break;

						case 'e':
							str += escapes[n][10];
							break;

						case 'i':
							str += escapes[n][11];
							break;

						case 'n':
							str += escapes[n][12];
							break;

						case 'o':
							str += escapes[n][13];
							break;

						case 'u':
							str += escapes[n][14];
							break;

						case 'y':
							str += escapes[n][15];
							break;

						case ' ':
							str += escapes[n][16];
							break;

						default:
							str += '?';
						};

					val_ptr++;
				}
			else
				{
					str += *val_ptr++;
				}
		}
}

//////////////////////////////////////////////////////////////////////
// Print raw data value
//////////////////////////////////////////////////////////////////////

void QStringSyntax::PrintRaw( CString &str, char *ldap_val_ptr, unsigned long len )
{
  str = ldap_val_ptr;
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

int QStringSyntax::Compare( QValue &v1, QValue &v2 )
{
  return _stricmp( v1.Value(), v2.Value() );
}
